=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2010-07-08 11:47:56 +0000
+++ bzrlib/builtins.py	2010-07-08 12:36:29 +0000
@@ -2717,7 +2717,7 @@
             if not globbing.Globster.is_pattern_valid(p):
                 bad_patterns.append(p)
         if bad_patterns:
-            msg = 'Invalid ignore patterns found.'
+            msg = 'Invalid ignore pattern(s) found.'
             pats = ''
             for p in bad_patterns:
                 pats += ('\n  %s' % p)

=== modified file 'bzrlib/globbing.py'
--- bzrlib/globbing.py	2010-07-08 11:57:22 +0000
+++ bzrlib/globbing.py	2010-07-08 12:36:29 +0000
@@ -203,20 +203,18 @@
 
     def __init__(self, patterns):
         self._regex_patterns = []
-        path_patterns = []
-        base_patterns = []
-        ext_patterns = []
         pattern_lists = {
-            Globster.TYPE_FULLPATH : path_patterns,
-            Globster.TYPE_EXTENSION : ext_patterns,
-            Globster.TYPE_BASENAME : base_patterns,
+            Globster.TYPE_FULLPATH : [],
+            Globster.TYPE_EXTENSION : [],
+            Globster.TYPE_BASENAME : [],
         }
         for pat in patterns:
             pat = normalize_pattern(pat)
             pattern_lists[Globster.identify(pat)].append(pat)
-        for k, v in pattern_lists.iteritems():
-            self._add_patterns(v, Globster.translators[k],
-                Globster.prefixes[k])
+        for pattern_type, patterns in pattern_lists.iteritems():
+            self._add_patterns(patterns,
+                Globster.translators[pattern_type],
+                Globster.prefixes[pattern_type])
 
     def _add_patterns(self, patterns, translator, prefix=''):
         while patterns:

=== modified file 'bzrlib/tests/blackbox/test_ignore.py'
--- bzrlib/tests/blackbox/test_ignore.py	2010-06-11 07:32:12 +0000
+++ bzrlib/tests/blackbox/test_ignore.py	2010-07-08 12:36:29 +0000
@@ -162,3 +162,19 @@
         tree = self.make_branch_and_tree('a')
         self.run_bzr(['ignore', '--directory=a', 'README'])
         self.check_file_contents('a/.bzrignore', 'README\n')
+
+    def test_ignored_invalid_pattern(self):
+        """Ensure graceful handling for invalid ignore pattern.
+
+        Test case for #300062.
+        Invalid pattern should show clear error message.
+        Invalid pattern should not be added to .bzrignore file.
+        """
+        tree = self.make_branch_and_tree('.')
+        out, err = self.run_bzr(['ignore', 'RE:*.cpp', 'foo', 'RE:['], 3)
+        self.assertEqual(out, '')
+        self.assertContainsRe(err,
+            'Invalid ignore pattern.*RE:\*\.cpp.*RE:\[', re.DOTALL)
+        self.assertNotContainsRe(err, 'foo', re.DOTALL)
+        self.assertFalse(os.path.isfile('.bzrignore'))
+

